@@ -31,10 +31,10 @@ __export(page_exports, {
   InitScript: () => InitScript,
   Page: () => Page,
   PageBinding: () => PageBinding,
-  Worker: () => Worker
+  Worker: () => Worker,
+  WorkerEvent: () => WorkerEvent
 });
 module.exports = __toCommonJS(page_exports);
-var accessibility = __toESM(require("./accessibility"));
 var import_browserContext = require("./browserContext");
 var import_console = require("./console");
 var import_errors = require("./errors");
@@ -54,13 +54,30 @@ var import_manualPromise = require("../utils/isomorphic/manualPromise");
 var import_utilityScriptSerializers = require("../utils/isomorphic/utilityScriptSerializers");
 var import_callLog = require("./callLog");
 var rawBindingsControllerSource = __toESM(require("../generated/bindingsControllerSource"));
+var import_screencast = require("./screencast");
+const PageEvent = {
+  Close: "close",
+  Crash: "crash",
+  Download: "download",
+  EmulatedSizeChanged: "emulatedsizechanged",
+  FileChooser: "filechooser",
+  FrameAttached: "frameattached",
+  FrameDetached: "framedetached",
+  InternalFrameNavigatedToNewDocument: "internalframenavigatedtonewdocument",
+  LocatorHandlerTriggered: "locatorhandlertriggered",
+  ScreencastFrame: "screencastframe",
+  Video: "video",
+  WebSocket: "websocket",
+  Worker: "worker"
+};
 class Page extends import_instrumentation.SdkObject {
   constructor(delegate, browserContext) {
     super(browserContext, "page");
     this._closedState = "open";
     this._closedPromise = new import_manualPromise.ManualPromise();
     this._initializedPromise = new import_manualPromise.ManualPromise();
-    this._eventsToEmitAfterInitialized = [];
+    this._consoleMessages = [];
+    this._pageErrors = [];
     this._crashed = false;
     this.openScope = new import_utils.LongStandingScope();
     this._emulatedMedia = {};
@@ -70,63 +87,47 @@ class Page extends import_instrumentation.SdkObject {
     this._workers = /* @__PURE__ */ new Map();
     this.requestInterceptors = [];
     this.video = null;
-    this._isServerSideOnly = false;
     this._locatorHandlers = /* @__PURE__ */ new Map();
     this._lastLocatorHandlerUid = 0;
     this._locatorHandlerRunningCounter = 0;
-    // Aiming at 25 fps by default - each frame is 40ms, but we give some slack with 35ms.
-    // When throttling for tracing, 200ms between frames, except for 10 frames around the action.
-    this._frameThrottler = new FrameThrottler(10, 35, 200);
-    this.lastSnapshotFrameIds = [];
+    this._networkRequests = [];
     this.attribution.page = this;
     this.delegate = delegate;
     this.browserContext = browserContext;
-    this.accessibility = new accessibility.Accessibility(delegate.getAccessibilityTree.bind(delegate));
     this.keyboard = new input.Keyboard(delegate.rawKeyboard, this);
     this.mouse = new input.Mouse(delegate.rawMouse, this);
     this.touchscreen = new input.Touchscreen(delegate.rawTouchscreen, this);
     this.screenshotter = new import_screenshotter.Screenshotter(this);
     this.frameManager = new frames.FrameManager(this);
+    this.screencast = new import_screencast.Screencast(this);
     if (delegate.pdf)
       this.pdf = delegate.pdf.bind(delegate);
     this.coverage = delegate.coverage ? delegate.coverage() : null;
+    this.isStorageStatePage = browserContext.isCreatingStorageStatePage();
   }
   static {
-    this.Events = {
-      Close: "close",
-      Crash: "crash",
-      Download: "download",
-      EmulatedSizeChanged: "emulatedsizechanged",
-      FileChooser: "filechooser",
-      FrameAttached: "frameattached",
-      FrameDetached: "framedetached",
-      InternalFrameNavigatedToNewDocument: "internalframenavigatedtonewdocument",
-      LocatorHandlerTriggered: "locatorhandlertriggered",
-      ScreencastFrame: "screencastframe",
-      Video: "video",
-      WebSocket: "websocket",
-      Worker: "worker"
-    };
+    this.Events = PageEvent;
   }
-  async reportAsNew(opener, error = void 0, contextEvent = import_browserContext.BrowserContext.Events.Page) {
+  async reportAsNew(opener, error) {
     if (opener) {
       const openerPageOrError = await opener.waitForInitializedOrError();
       if (openerPageOrError instanceof Page && !openerPageOrError.isClosed())
         this._opener = openerPageOrError;
     }
-    this._markInitialized(error, contextEvent);
+    this._markInitialized(error);
   }
-  _markInitialized(error = void 0, contextEvent = import_browserContext.BrowserContext.Events.Page) {
+  _markInitialized(error = void 0) {
     if (error) {
       if (this.browserContext.isClosingOrClosed())
         return;
       this.frameManager.createDummyMainFrameIfNeeded();
     }
     this._initialized = error || this;
-    this.emitOnContext(contextEvent, this);
-    for (const { event, args } of this._eventsToEmitAfterInitialized)
-      this.browserContext.emit(event, ...args);
-    this._eventsToEmitAfterInitialized = [];
+    this.emitOnContext(import_browserContext.BrowserContext.Events.Page, this);
+    for (const pageError of this._pageErrors)
+      this.emitOnContext(import_browserContext.BrowserContext.Events.PageError, pageError, this);
+    for (const message of this._consoleMessages)
+      this.emitOnContext(import_browserContext.BrowserContext.Events.Console, message);
     if (this.isClosed())
       this.emit(Page.Events.Close);
     else
@@ -140,18 +141,10 @@ class Page extends import_instrumentation.SdkObject {
     return this._initializedPromise;
   }
   emitOnContext(event, ...args) {
-    if (this._isServerSideOnly)
+    if (this.isStorageStatePage)
       return;
     this.browserContext.emit(event, ...args);
   }
-  emitOnContextOnceInitialized(event, ...args) {
-    if (this._isServerSideOnly)
-      return;
-    if (this._initialized)
-      this.browserContext.emit(event, ...args);
-    else
-      this._eventsToEmitAfterInitialized.push({ event, args });
-  }
   async resetForReuse(progress) {
     await this.mainFrame().gotoImpl(progress, "about:blank", {});
     this._emulatedSize = void 0;
@@ -166,17 +159,17 @@ class Page extends import_instrumentation.SdkObject {
   }
   _didClose() {
     this.frameManager.dispose();
-    this._frameThrottler.dispose();
+    this.screencast.stopFrameThrottler();
     (0, import_utils.assert)(this._closedState !== "closed", "Page closed twice");
     this._closedState = "closed";
     this.emit(Page.Events.Close);
     this._closedPromise.resolve();
     this.instrumentation.onPageClose(this);
-    this.openScope.close(new import_errors.TargetClosedError());
+    this.openScope.close(new import_errors.TargetClosedError(this.closeReason()));
   }
   _didCrash() {
     this.frameManager.dispose();
-    this._frameThrottler.dispose();
+    this.screencast.stopFrameThrottler();
     this.emit(Page.Events.Crash);
     this._crashed = true;
     this.instrumentation.onPageClose(this);
@@ -213,10 +206,14 @@ class Page extends import_instrumentation.SdkObject {
     await progress.race(this.browserContext.exposePlaywrightBindingIfNeeded());
     const binding = new PageBinding(name, playwrightBinding, needsHandle);
     this._pageBindings.set(name, binding);
-    progress.cleanupWhenAborted(() => this._pageBindings.delete(name));
-    await progress.race(this.delegate.addInitScript(binding.initScript));
-    await progress.race(this.safeNonStallingEvaluateInAllFrames(binding.initScript.source, "main"));
-    return binding;
+    try {
+      await progress.race(this.delegate.addInitScript(binding.initScript));
+      await progress.race(this.safeNonStallingEvaluateInAllFrames(binding.initScript.source, "main"));
+      return binding;
+    } catch (error) {
+      this._pageBindings.delete(name);
+      throw error;
+    }
   }
   async removeExposedBindings(bindings) {
     bindings = bindings.filter((binding) => this._pageBindings.get(binding.name) === binding);
@@ -229,29 +226,54 @@ class Page extends import_instrumentation.SdkObject {
   }
   async setExtraHTTPHeaders(progress, headers) {
     const oldHeaders = this._extraHTTPHeaders;
-    this._extraHTTPHeaders = headers;
-    progress.cleanupWhenAborted(async () => {
+    try {
+      this._extraHTTPHeaders = headers;
+      await progress.race(this.delegate.updateExtraHTTPHeaders());
+    } catch (error) {
       this._extraHTTPHeaders = oldHeaders;
-      await this.delegate.updateExtraHTTPHeaders();
-    });
-    await progress.race(this.delegate.updateExtraHTTPHeaders());
+      this.delegate.updateExtraHTTPHeaders().catch(() => {
+      });
+      throw error;
+    }
   }
   extraHTTPHeaders() {
     return this._extraHTTPHeaders;
   }
+  addNetworkRequest(request) {
+    this._networkRequests.push(request);
+    ensureArrayLimit(this._networkRequests, 100);
+  }
+  networkRequests() {
+    return this._networkRequests;
+  }
   async onBindingCalled(payload, context) {
     if (this._closedState === "closed")
       return;
     await PageBinding.dispatch(this, payload, context);
   }
-  addConsoleMessage(type, args, location, text) {
-    const message = new import_console.ConsoleMessage(this, type, text, args, location);
+  addConsoleMessage(worker, type, args, location, text) {
+    const message = new import_console.ConsoleMessage(this, worker, type, text, args, location);
     const intercepted = this.frameManager.interceptConsoleMessage(message);
     if (intercepted) {
       args.forEach((arg) => arg.dispose());
       return;
     }
-    this.emitOnContextOnceInitialized(import_browserContext.BrowserContext.Events.Console, message);
+    this._consoleMessages.push(message);
+    ensureArrayLimit(this._consoleMessages, 200);
+    if (this._initialized)
+      this.emitOnContext(import_browserContext.BrowserContext.Events.Console, message);
+  }
+  consoleMessages() {
+    return this._consoleMessages;
+  }
+  addPageError(pageError) {
+    this._pageErrors.push(pageError);
+    ensureArrayLimit(this._pageErrors, 200);
+    if (this._initialized)
+      this.emitOnContext(import_browserContext.BrowserContext.Events.PageError, pageError, this);
+  }
+  pageErrors() {
+    return this._pageErrors;
   }
   async reload(progress, options) {
     return this.mainFrame().raceNavigationAction(progress, async () => {
@@ -327,6 +349,8 @@ class Page extends import_instrumentation.SdkObject {
     await this._performWaitForNavigationCheck(progress);
   }
   async _performWaitForNavigationCheck(progress) {
+    if (process.env.PLAYWRIGHT_SKIP_NAVIGATION_CHECK)
+      return;
     const mainFrame = this.frameManager.mainFrame();
     if (!mainFrame || !mainFrame.pendingDocument())
       return;
@@ -369,10 +393,6 @@ class Page extends import_instrumentation.SdkObject {
   }
   async emulateMedia(progress, options) {
     const oldEmulatedMedia = { ...this._emulatedMedia };
-    progress.cleanupWhenAborted(async () => {
-      this._emulatedMedia = oldEmulatedMedia;
-      await this.delegate.updateEmulateMedia();
-    });
     if (options.media !== void 0)
       this._emulatedMedia.media = options.media;
     if (options.colorScheme !== void 0)
@@ -383,7 +403,14 @@ class Page extends import_instrumentation.SdkObject {
       this._emulatedMedia.forcedColors = options.forcedColors;
     if (options.contrast !== void 0)
       this._emulatedMedia.contrast = options.contrast;
-    await progress.race(this.delegate.updateEmulateMedia());
+    try {
+      await progress.race(this.delegate.updateEmulateMedia());
+    } catch (error) {
+      this._emulatedMedia = oldEmulatedMedia;
+      this.delegate.updateEmulateMedia().catch(() => {
+      });
+      throw error;
+    }
   }
   emulatedMedia() {
     const contextOptions = this.browserContext._options;
@@ -397,12 +424,15 @@ class Page extends import_instrumentation.SdkObject {
   }
   async setViewportSize(progress, viewportSize) {
     const oldEmulatedSize = this._emulatedSize;
-    progress.cleanupWhenAborted(async () => {
+    try {
+      this._setEmulatedSize({ viewport: { ...viewportSize }, screen: { ...viewportSize } });
+      await progress.race(this.delegate.updateEmulatedViewportSize());
+    } catch (error) {
       this._emulatedSize = oldEmulatedSize;
-      await this.delegate.updateEmulatedViewportSize();
-    });
-    this._setEmulatedSize({ viewport: { ...viewportSize }, screen: { ...viewportSize } });
-    await progress.race(this.delegate.updateEmulatedViewportSize());
+      this.delegate.updateEmulatedViewportSize().catch(() => {
+      });
+      throw error;
+    }
   }
   setEmulatedSizeFromWindowOpen(emulatedSize) {
     this._setEmulatedSize(emulatedSize);
@@ -423,8 +453,13 @@ class Page extends import_instrumentation.SdkObject {
   async addInitScript(progress, source) {
     const initScript = new InitScript(source);
     this.initScripts.push(initScript);
-    progress.cleanupWhenAborted(() => this.removeInitScripts([initScript]));
-    await progress.race(this.delegate.addInitScript(initScript));
+    try {
+      await progress.race(this.delegate.addInitScript(initScript));
+    } catch (error) {
+      this.removeInitScripts([initScript]).catch(() => {
+      });
+      throw error;
+    }
     return initScript;
   }
   async removeInitScripts(initScripts) {
@@ -478,20 +513,6 @@ class Page extends import_instrumentation.SdkObject {
         intermediateResult = { errorMessage: comparatorResult.errorMessage, diff: comparatorResult.diff, actual, previous };
       return false;
     };
-    const handleError = (e) => {
-      if (js.isJavaScriptErrorInEvaluate(e) || (0, import_selectorParser.isInvalidSelectorError)(e))
-        throw e;
-      let errorMessage = e.message;
-      if (e instanceof import_errors.TimeoutError && intermediateResult?.previous)
-        errorMessage = `Failed to take two consecutive stable screenshots.`;
-      return {
-        log: (0, import_callLog.compressCallLog)(e.message ? [...progress.metadata.log, e.message] : progress.metadata.log),
-        ...intermediateResult,
-        errorMessage,
-        timedOut: e instanceof import_errors.TimeoutError
-      };
-    };
-    progress.legacySetErrorHandler(handleError);
     try {
       let actual;
       let previous;
@@ -538,7 +559,17 @@ class Page extends import_instrumentation.SdkObject {
       }
       throw new Error(intermediateResult.errorMessage);
     } catch (e) {
-      return handleError(e);
+      if (js.isJavaScriptErrorInEvaluate(e) || (0, import_selectorParser.isInvalidSelectorError)(e))
+        throw e;
+      let errorMessage = e.message;
+      if (e instanceof import_errors.TimeoutError && intermediateResult?.previous)
+        errorMessage = `Failed to take two consecutive stable screenshots.`;
+      return {
+        log: (0, import_callLog.compressCallLog)(e.message ? [...progress.metadata.log, e.message] : progress.metadata.log),
+        ...intermediateResult,
+        errorMessage,
+        timedOut: e instanceof import_errors.TimeoutError
+      };
     }
   }
   async screenshot(progress, options) {
@@ -548,10 +579,11 @@ class Page extends import_instrumentation.SdkObject {
     if (this._closedState === "closed")
       return;
     if (options.reason)
-      this.closeReason = options.reason;
+      this._closeReason = options.reason;
     const runBeforeUnload = !!options.runBeforeUnload;
     if (this._closedState !== "closing") {
-      this._closedState = "closing";
+      if (!runBeforeUnload)
+        this._closedState = "closing";
       await this.delegate.closePage(runBeforeUnload).catch((e) => import_debugLogger.debugLogger.log("error", e));
     }
     if (!runBeforeUnload)
@@ -610,16 +642,6 @@ class Page extends import_instrumentation.SdkObject {
   getBinding(name) {
     return this._pageBindings.get(name) || this.browserContext._pageBindings.get(name);
   }
-  setScreencastOptions(options) {
-    this.delegate.setScreencastOptions(options).catch((e) => import_debugLogger.debugLogger.log("error", e));
-    this._frameThrottler.setThrottlingEnabled(!!options);
-  }
-  throttleScreencastFrameAck(ack) {
-    this._frameThrottler.ack(ack);
-  }
-  temporarilyDisableTracingScreencastThrottling() {
-    this._frameThrottler.recharge();
-  }
   async safeNonStallingEvaluateInAllFrames(expression, world, options = {}) {
     await Promise.all(this.frames().map(async (frame) => {
       try {
@@ -634,35 +656,37 @@ class Page extends import_instrumentation.SdkObject {
     await Promise.all(this.frames().map((frame) => frame.hideHighlight().catch(() => {
     })));
   }
-  markAsServerSideOnly() {
-    this._isServerSideOnly = true;
-  }
-  async snapshotForAI(progress) {
-    this.lastSnapshotFrameIds = [];
-    const snapshot = await snapshotFrameForAI(progress, this.mainFrame(), 0, this.lastSnapshotFrameIds);
-    return snapshot.join("\n");
+  async snapshotForAI(progress, options = {}) {
+    const snapshot = await snapshotFrameForAI(progress, this.mainFrame(), options);
+    return { full: snapshot.full.join("\n"), incremental: snapshot.incremental?.join("\n") };
   }
 }
+const WorkerEvent = {
+  Close: "close"
+};
 class Worker extends import_instrumentation.SdkObject {
   constructor(parent, url) {
     super(parent, "worker");
+    this._executionContextPromise = new import_manualPromise.ManualPromise();
+    this._workerScriptLoaded = false;
     this.existingExecutionContext = null;
     this.openScope = new import_utils.LongStandingScope();
     this.url = url;
-    this._executionContextCallback = () => {
-    };
-    this._executionContextPromise = new Promise((x) => this._executionContextCallback = x);
   }
   static {
-    this.Events = {
-      Close: "close"
-    };
+    this.Events = WorkerEvent;
   }
   createExecutionContext(delegate) {
     this.existingExecutionContext = new js.ExecutionContext(this, delegate, "worker");
-    this._executionContextCallback(this.existingExecutionContext);
+    if (this._workerScriptLoaded)
+      this._executionContextPromise.resolve(this.existingExecutionContext);
     return this.existingExecutionContext;
   }
+  workerScriptLoaded() {
+    this._workerScriptLoaded = true;
+    if (this.existingExecutionContext)
+      this._executionContextPromise.resolve(this.existingExecutionContext);
+  }
   didClose() {
     if (this.existingExecutionContext)
       this.existingExecutionContext.contextDestroyed("Worker was closed");
@@ -731,66 +755,17 @@ class InitScript {
     })();`;
   }
 }
-class FrameThrottler {
-  constructor(nonThrottledFrames, defaultInterval, throttlingInterval) {
-    this._acks = [];
-    this._throttlingEnabled = false;
-    this._nonThrottledFrames = nonThrottledFrames;
-    this._budget = nonThrottledFrames;
-    this._defaultInterval = defaultInterval;
-    this._throttlingInterval = throttlingInterval;
-    this._tick();
-  }
-  dispose() {
-    if (this._timeoutId) {
-      clearTimeout(this._timeoutId);
-      this._timeoutId = void 0;
-    }
-  }
-  setThrottlingEnabled(enabled) {
-    this._throttlingEnabled = enabled;
-  }
-  recharge() {
-    for (const ack of this._acks)
-      ack();
-    this._acks = [];
-    this._budget = this._nonThrottledFrames;
-    if (this._timeoutId) {
-      clearTimeout(this._timeoutId);
-      this._tick();
-    }
-  }
-  ack(ack) {
-    if (!this._timeoutId) {
-      ack();
-      return;
-    }
-    this._acks.push(ack);
-  }
-  _tick() {
-    const ack = this._acks.shift();
-    if (ack) {
-      --this._budget;
-      ack();
-    }
-    if (this._throttlingEnabled && this._budget <= 0) {
-      this._timeoutId = setTimeout(() => this._tick(), this._throttlingInterval);
-    } else {
-      this._timeoutId = setTimeout(() => this._tick(), this._defaultInterval);
-    }
-  }
-}
-async function snapshotFrameForAI(progress, frame, frameOrdinal, frameIds) {
+async function snapshotFrameForAI(progress, frame, options = {}) {
   const snapshot = await frame.retryWithProgressAndTimeouts(progress, [1e3, 2e3, 4e3, 8e3], async (continuePolling) => {
     try {
       const context = await progress.race(frame._utilityContext());
       const injectedScript = await progress.race(context.injectedScript());
-      const snapshotOrRetry = await progress.race(injectedScript.evaluate((injected, refPrefix) => {
+      const snapshotOrRetry = await progress.race(injectedScript.evaluate((injected, options2) => {
         const node = injected.document.body;
         if (!node)
           return true;
-        return injected.ariaSnapshot(node, { forAI: true, refPrefix });
-      }, frameOrdinal ? "f" + frameOrdinal : ""));
+        return injected.incrementalAriaSnapshot(node, { mode: "ai", ...options2 });
+      }, { refPrefix: frame.seq ? "f" + frame.seq : "", track: options.track, doNotRenderActive: options.doNotRenderActive }));
       if (snapshotOrRetry === true)
         return continuePolling;
       return snapshotOrRetry;
@@ -800,38 +775,56 @@ async function snapshotFrameForAI(progress, frame, frameOrdinal, frameIds) {
       return continuePolling;
     }
   });
-  const lines = snapshot.split("\n");
-  const result = [];
-  for (const line of lines) {
-    const match = line.match(/^(\s*)- iframe (?:\[active\] )?\[ref=(.*)\]/);
+  const childSnapshotPromises = snapshot.iframeRefs.map((ref) => snapshotFrameRefForAI(progress, frame, ref, options));
+  const childSnapshots = await Promise.all(childSnapshotPromises);
+  const full = [];
+  let incremental;
+  if (snapshot.incremental !== void 0) {
+    incremental = snapshot.incremental.split("\n");
+    for (let i = 0; i < snapshot.iframeRefs.length; i++) {
+      const childSnapshot = childSnapshots[i];
+      if (childSnapshot.incremental)
+        incremental.push(...childSnapshot.incremental);
+      else if (childSnapshot.full.length)
+        incremental.push("- <changed> iframe [ref=" + snapshot.iframeRefs[i] + "]:", ...childSnapshot.full.map((l) => "  " + l));
+    }
+  }
+  for (const line of snapshot.full.split("\n")) {
+    const match = line.match(/^(\s*)- iframe (?:\[active\] )?\[ref=([^\]]*)\]/);
     if (!match) {
-      result.push(line);
+      full.push(line);
       continue;
     }
     const leadingSpace = match[1];
     const ref = match[2];
-    const frameSelector = `aria-ref=${ref} >> internal:control=enter-frame`;
-    const frameBodySelector = `${frameSelector} >> body`;
-    const child = await progress.race(frame.selectors.resolveFrameForSelector(frameBodySelector, { strict: true }));
-    if (!child) {
-      result.push(line);
-      continue;
-    }
-    const frameOrdinal2 = frameIds.length + 1;
-    frameIds.push(child.frame._id);
-    try {
-      const childSnapshot = await snapshotFrameForAI(progress, child.frame, frameOrdinal2, frameIds);
-      result.push(line + ":", ...childSnapshot.map((l) => leadingSpace + "  " + l));
-    } catch {
-      result.push(line);
-    }
+    const childSnapshot = childSnapshots[snapshot.iframeRefs.indexOf(ref)] ?? { full: [] };
+    full.push(childSnapshot.full.length ? line + ":" : line);
+    full.push(...childSnapshot.full.map((l) => leadingSpace + "  " + l));
   }
-  return result;
+  return { full, incremental };
+}
+async function snapshotFrameRefForAI(progress, parentFrame, frameRef, options) {
+  const frameSelector = `aria-ref=${frameRef} >> internal:control=enter-frame`;
+  const frameBodySelector = `${frameSelector} >> body`;
+  const child = await progress.race(parentFrame.selectors.resolveFrameForSelector(frameBodySelector, { strict: true }));
+  if (!child)
+    return { full: [] };
+  try {
+    return await snapshotFrameForAI(progress, child.frame, options);
+  } catch {
+    return { full: [] };
+  }
+}
+function ensureArrayLimit(array, limit) {
+  if (array.length > limit)
+    return array.splice(0, limit / 10);
+  return [];
 }
 // Annotate the CommonJS export names for ESM import in node:
 0 && (module.exports = {
   InitScript,
   Page,
   PageBinding,
-  Worker
+  Worker,
+  WorkerEvent
 });
